@@ -291,6 +291,8 @@ po::options_description get_fscp_options()
 	("fscp.never_contact", po::value<std::vector<asiotap::ip_network_address> >()->multitoken()->zero_tokens()->default_value(std::vector<asiotap::ip_network_address>(), ""), "A network address to avoid when dynamically contacting hosts.")
 	("fscp.cipher_suite_capability", po::value<std::vector<fscp::cipher_suite_type> >()->multitoken()->zero_tokens()->default_value(fscp::get_default_cipher_suites(), ""), "A cipher suite to allow.")
 	("fscp.elliptic_curve_capability", po::value<std::vector<fscp::elliptic_curve_type> >()->multitoken()->zero_tokens()->default_value(fscp::get_default_elliptic_curves(), ""), "A elliptic curve to allow.")
+	("fscp.upnp_enabled", po::value<bool>()->default_value(true, "yes"), "Enable UPnP.")
+	("fscp.max_unauthenticated_messages_per_second", po::value<size_t>()->default_value(1, "1"), "Maximum unauthenticated messages from one host per second.")
 	;
 
 	return result;
@@ -333,6 +335,7 @@ po::options_description get_tap_adapter_options()
 	("tap_adapter.mss_override", po::value<fl::mss_type>()->default_value(fl::mss_type()), "The MSS override.")
 	("tap_adapter.metric", po::value<fl::metric_type>()->default_value(fl::auto_metric_type()), "The metric of the tap adapter.")
 	("tap_adapter.ipv4_address_prefix_length", po::value<asiotap::ipv4_network_address>(), "The tap adapter IPv4 address and prefix length.")
+	("tap_adapter.ipv4_dhcp", po::value<bool>()->default_value(false), "The tap adapter IPv4 DHCP status.")
 	("tap_adapter.ipv6_address_prefix_length", po::value<asiotap::ipv6_network_address>(), "The tap adapter IPv6 address and prefix length.")
 	("tap_adapter.remote_ipv4_address", po::value<asiotap::ipv4_network_address>(), "The tap adapter IPv4 remote address.")
 	("tap_adapter.arp_proxy_enabled", po::value<bool>()->default_value(false), "Whether to enable the ARP proxy.")
@@ -462,6 +465,8 @@ void setup_configuration(const fscp::logger& logger, fl::configuration& configur
 	configuration.fscp.never_contact_list = vm["fscp.never_contact"].as<std::vector<asiotap::ip_network_address>>();
 	configuration.fscp.cipher_suite_capabilities = vm["fscp.cipher_suite_capability"].as<std::vector<fscp::cipher_suite_type>>();
 	configuration.fscp.elliptic_curve_capabilities = vm["fscp.elliptic_curve_capability"].as<std::vector<fscp::elliptic_curve_type>>();
+	configuration.fscp.upnp_enabled = vm["fscp.upnp_enabled"].as<bool>();
+	configuration.fscp.max_unauthenticated_messages_per_second = vm["fscp.max_unauthenticated_messages_per_second"].as<size_t>();
 
 	// Security options
 	const std::string passphrase = vm["security.passphrase"].as<std::string>();
@@ -534,6 +539,11 @@ void setup_configuration(const fscp::logger& logger, fl::configuration& configur
 		configuration.tap_adapter.ipv4_address_prefix_length = vm["tap_adapter.ipv4_address_prefix_length"].as<asiotap::ipv4_network_address>();
 	}
 	
+	if (vm.count("tap_adapter.ipv4_dhcp"))
+	{
+		configuration.tap_adapter.ipv4_dhcp = vm["tap_adapter.ipv4_dhcp"].as<bool>();
+	}
+
 	if (vm.count("tap_adapter.ipv6_address_prefix_length"))
 	{
 		configuration.tap_adapter.ipv6_address_prefix_length = vm["tap_adapter.ipv6_address_prefix_length"].as<asiotap::ipv6_network_address>();
