@@ -56,6 +56,7 @@
 #ifdef LINUX
 
 #include <linux/if_tun.h>
+#include <sys/sysmacros.h>
 
 /**
  * \struct in6_ifreq
@@ -70,13 +71,15 @@ struct in6_ifreq
 	int ifr6_ifindex; /**< Interface index */
 };
 
-#elif defined(MACINTOSH) || defined(BSD)
+#elif defined(MACINTOSH) || defined(OS_BSD)
 
 /*
  * Note for Mac OS X users : you have to download and install the tun/tap driver from (http://tuntaposx.sourceforge.net).
  */
-
+#ifndef __NetBSD__
 #include <net/if_var.h>
+#endif
+
 #include <net/if_types.h>
 #include <net/if_dl.h>
 #include <net/if.h>
@@ -206,6 +209,7 @@ namespace asiotap
 						{
 							result[name] = name;
 						}
+						break;
 					}
 					case tap_adapter_layer::ip:
 					{
@@ -213,6 +217,7 @@ namespace asiotap
 						{
 							result[name] = name;
 						}
+						break;
 					}
 				}
 			}
@@ -230,6 +235,8 @@ namespace asiotap
 	{
 		ec = boost::system::error_code();
 
+		m_existing_tap = !_name.empty();
+
 #if defined(LINUX)
 		const std::string dev_name = (layer() == tap_adapter_layer::ethernet) ? "/dev/net/tap" : "/dev/net/tun";
 
@@ -237,13 +244,13 @@ namespace asiotap
 		{
 			if (errno != ENOENT)
 			{
-			// Unable to access the tap adapter yet it exists: this is an error.
+				// Unable to access the tap adapter yet it exists: this is an error.
 				ec = boost::system::error_code(errno, boost::system::system_category());
 
 				return;
 			}
 
-		// No tap found, create one.
+			// No tap found, create one.
 			if (::mknod(dev_name.c_str(), S_IFCHR | S_IRUSR | S_IWUSR, ::makedev(10, 200)) == -1)
 			{
 				ec = boost::system::error_code(errno, boost::system::system_category());
@@ -279,6 +286,7 @@ namespace asiotap
 		if (!_name.empty())
 		{
 			strncpy(ifr.ifr_name, _name.c_str(), IFNAMSIZ);
+			ifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 		}
 
 		// Set the parameters on the tun device.
@@ -300,12 +308,13 @@ namespace asiotap
 			struct ifreq netifr {};
 
 			std::strncpy(netifr.ifr_name, ifr.ifr_name, IFNAMSIZ);
+			netifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 
 #if defined(IFF_ONE_QUEUE) && defined(SIOCSIFTXQLEN)
 
 			netifr.ifr_qlen = 100; // 100 is the default value
 
-			if (::ioctl(socket.native_handle(), SIOCSIFTXQLEN, (void *)&netifr) < 0)
+			if (getuid() == 0 && ::ioctl(socket.native_handle(), SIOCSIFTXQLEN, (void *)&netifr) < 0)
 			{
 				ec = boost::system::error_code(errno, boost::system::system_category());
 
@@ -316,6 +325,7 @@ namespace asiotap
 			netifr = {};
 
 			std::strncpy(netifr.ifr_name, ifr.ifr_name, IFNAMSIZ);
+			netifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 #endif
 
 			// Get the interface hwaddr
@@ -378,8 +388,18 @@ namespace asiotap
 		char namebuf[256];
 		memset(namebuf, 0x00, sizeof(namebuf));
 
+#ifdef __NetBSD__
+		if (::devname_r(st.st_dev, S_IFCHR, namebuf, 255) != 0)
+		{
+#elif defined(__OpenBSD__)
+		char* n = ::devname(st.st_dev, S_IFCHR);
+		if(n)
+		{
+			strncpy(namebuf, n, 255);
+#else
 		if (::devname_r(st.st_dev, S_IFCHR, namebuf, 255) != NULL)
 		{
+#endif
 			set_name(namebuf);
 		}
 		else
@@ -470,7 +490,13 @@ namespace asiotap
 
 	void posix_tap_adapter::destroy_device(boost::system::error_code& ec)
 	{
-#if defined(MACINTOSH) || defined(BSD)
+		// do not attempt to destroy interface if non-root
+		if(getuid() != 0)
+		{
+			return;
+		}
+
+#if defined(MACINTOSH) || defined(OS_BSD)
 		descriptor_handler socket = open_socket(AF_INET, ec);
 
 		if (!socket.valid())
@@ -481,6 +507,7 @@ namespace asiotap
 		struct ifreq ifr {};
 
 		strncpy(ifr.ifr_name, name().c_str(), IFNAMSIZ);
+		ifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 
 		// Destroy the virtual tap device
 		if (ioctl(socket.native_handle(), SIOCIFDESTROY, &ifr) < 0)
@@ -499,13 +526,20 @@ namespace asiotap
 		struct ifreq netifr {};
 
 		strncpy(netifr.ifr_name, name().c_str(), IFNAMSIZ);
+		netifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 
-		// Set the interface UP
+		// Get the interface flags
 		if (::ioctl(socket.native_handle(), SIOCGIFFLAGS, static_cast<void*>(&netifr)) < 0)
 		{
 			throw boost::system::system_error(errno, boost::system::system_category());
 		}
 
+		// as non-root, assume that existing TAP is correctly configured
+		if (getuid() != 0 && m_existing_tap)
+		{
+			return;
+		}
+
 		if (connected)
 		{
 #ifdef MACINTOSH
@@ -525,6 +559,7 @@ namespace asiotap
 #endif
 		}
 
+		// Set the interface UP
 		if (::ioctl(socket.native_handle(), SIOCSIFFLAGS, static_cast<void*>(&netifr)) < 0)
 		{
 			throw boost::system::system_error(errno, boost::system::system_category());
@@ -575,7 +610,7 @@ namespace asiotap
 					struct sockaddr_in6* sai = reinterpret_cast<struct sockaddr_in6*>(ifa->ifa_addr);
 
 					boost::asio::ip::address_v6::bytes_type bytes;
-					memcpy(bytes.data(), &sai->sin6_addr, bytes.size());
+					memcpy(bytes.data(), &sai->sin6_addr.s6_addr, bytes.size());
 
 					boost::asio::ip::address_v6 address(bytes);
 
@@ -598,6 +633,12 @@ namespace asiotap
 
 	void posix_tap_adapter::configure(const configuration_type& configuration)
 	{
+		// as non-root, assume that existing TAP is correctly configured
+		if(getuid() != 0 && m_existing_tap)
+		{
+			return;
+		}
+
 		if (configuration.ipv4.network_address)
 		{
 			if (layer() == tap_adapter_layer::ethernet)
@@ -631,6 +672,7 @@ namespace asiotap
 		struct ifreq netifr {};
 
 		strncpy(netifr.ifr_name, name().c_str(), IFNAMSIZ);
+		netifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 
 		if (::ioctl(socket.native_handle(), SIOCGIFMTU, (void*)&netifr) >= 0)
 		{
@@ -649,6 +691,7 @@ namespace asiotap
 		struct ifreq netifr {};
 
 		strncpy(netifr.ifr_name, name().c_str(), IFNAMSIZ);
+		netifr.ifr_name[IFNAMSIZ - 1] = 0x00;
 
 		netifr.ifr_mtu = _mtu;
 
@@ -669,11 +712,12 @@ namespace asiotap
 
 		ifreq ifr_a {};
 
-		std::strncpy(ifr_a.ifr_name, name().c_str(), IFNAMSIZ - 1);
+		std::strncpy(ifr_a.ifr_name, name().c_str(), IFNAMSIZ);
+		ifr_a.ifr_name[IFNAMSIZ - 1] = 0x00;
 
 		sockaddr_in* ifr_a_addr = reinterpret_cast<sockaddr_in*>(&ifr_a.ifr_addr);
 		ifr_a_addr->sin_family = AF_INET;
-#ifdef BSD
+#ifdef OS_BSD
 		ifr_a_addr->sin_len = sizeof(sockaddr_in);
 #endif
 		std::memcpy(&ifr_a_addr->sin_addr.s_addr, address.to_bytes().data(), address.to_bytes().size());
@@ -694,10 +738,11 @@ namespace asiotap
 		{
 			ifreq ifr_n {};
 
-			std::strncpy(ifr_n.ifr_name, name().c_str(), IFNAMSIZ - 1);
+			std::strncpy(ifr_n.ifr_name, name().c_str(), IFNAMSIZ);
+			ifr_n.ifr_name[IFNAMSIZ - 1] = 0x00;
 			sockaddr_in* ifr_n_addr = reinterpret_cast<sockaddr_in*>(&ifr_n.ifr_addr);
 			ifr_n_addr->sin_family = AF_INET;
-#ifdef BSD
+#ifdef OS_BSD
 			ifr_n_addr->sin_len = sizeof(sockaddr_in);
 #endif
 			ifr_n_addr->sin_addr.s_addr = htonl((0xFFFFFFFF >> (32 - prefix_len)) << (32 - prefix_len));
@@ -734,17 +779,17 @@ namespace asiotap
 		}
 
 		in6_ifreq ifr {};
-		std::memcpy(&ifr.ifr6_addr, address.to_bytes().data(), address.to_bytes().size());
+		std::memcpy(&ifr.ifr6_addr.s6_addr, address.to_bytes().data(), address.to_bytes().size());
 		ifr.ifr6_prefixlen = prefix_len;
 		ifr.ifr6_ifindex = if_index;
 
 		if (::ioctl(socket.native_handle(), SIOCSIFADDR, &ifr) < 0)
-#elif defined(MACINTOSH) || defined(BSD)
+#elif defined(MACINTOSH) || defined(OS_BSD)
 		in6_aliasreq iar {};
 		std::memcpy(iar.ifra_name, name().c_str(), name().length());
 		reinterpret_cast<sockaddr_in6*>(&iar.ifra_addr)->sin6_family = AF_INET6;
 		reinterpret_cast<sockaddr_in6*>(&iar.ifra_prefixmask)->sin6_family = AF_INET6;
-		std::memcpy(&reinterpret_cast<sockaddr_in6*>(&iar.ifra_addr)->sin6_addr, address.to_bytes().data(), address.to_bytes().size());
+		std::memcpy(&reinterpret_cast<sockaddr_in6*>(&iar.ifra_addr)->sin6_addr.s6_addr, address.to_bytes().data(), address.to_bytes().size());
 		std::memset(reinterpret_cast<sockaddr_in6*>(&iar.ifra_prefixmask)->sin6_addr.s6_addr, 0xFF, prefix_len / 8);
 		reinterpret_cast<sockaddr_in6*>(&iar.ifra_prefixmask)->sin6_addr.s6_addr[prefix_len / 8] = (0xFF << (8 - (prefix_len % 8)));
 		iar.ifra_lifetime.ia6t_pltime = 0xFFFFFFFF;
@@ -792,10 +837,11 @@ namespace asiotap
 
 		ifreq ifr_d {};
 
-		std::strncpy(ifr_d.ifr_name, name().c_str(), IFNAMSIZ - 1);
+		std::strncpy(ifr_d.ifr_name, name().c_str(), IFNAMSIZ);
+		ifr_d.ifr_name[IFNAMSIZ - 1] = 0x00;
 		sockaddr_in* ifr_dst_addr = reinterpret_cast<sockaddr_in*>(&ifr_d.ifr_dstaddr);
 		ifr_dst_addr->sin_family = AF_INET;
-#ifdef BSD
+#ifdef OS_BSD
 		ifr_dst_addr->sin_len = sizeof(sockaddr_in);
 #endif
 		std::memcpy(&ifr_dst_addr->sin_addr.s_addr, remote_address.to_bytes().data(), remote_address.to_bytes().size());
