@@ -565,10 +565,17 @@ namespace cryptoplus
 		{
 			return EVP_PKEY_size(ptr().get());
 		}
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+		inline int pkey::type() const
+		{
+			return EVP_PKEY_base_id(ptr().get());
+		}
+#else
 		inline int pkey::type() const
 		{
 			return EVP_PKEY_type(ptr()->type);
 		}
+#endif
 		inline bool pkey::is_rsa() const
 		{
 			return (type() == EVP_PKEY_RSA);
