@@ -316,6 +316,16 @@ namespace freelan
 		 * \brief The list of allowed elliptic curves.
 		 */
 		fscp::elliptic_curve_list_type elliptic_curve_capabilities;
+
+		/**
+		 * \brief State of UPnP.
+		 */
+		bool upnp_enabled;
+
+		/*
+		 * \brief Maximum HELLO/PRESENTATION message from one host per second.
+		 */
+		size_t max_unauthenticated_messages_per_second;
 	};
 
 	/**
@@ -457,6 +467,11 @@ namespace freelan
 		 */
 		asiotap::ipv4_network_address ipv4_address_prefix_length;
 
+		/**
+		 * \brief The IPv4 tap adapter DHCP status.
+		 */
+		bool ipv4_dhcp;
+
 		/**
 		 * \brief The IPv6 tap adapter address.
 		 */
