@@ -42,6 +42,8 @@
  * \author Julien KAUFFMANN <julien.kauffmann@freelan.org>
  * \brief The freelan core class.
  */
+//Use the behavior of Boost from bevor 1.63
+#define BOOST_NO_CXX11_UNIFIED_INITIALIZATION_SYNTAX
 
 #include "core.hpp"
 
@@ -630,6 +632,8 @@ namespace freelan
 		{
 			m_fscp_server->set_cipher_suites(m_configuration.fscp.cipher_suite_capabilities);
 			m_fscp_server->set_elliptic_curves(m_configuration.fscp.elliptic_curve_capabilities);
+			m_fscp_server->set_hello_max_per_second(m_configuration.fscp.max_unauthenticated_messages_per_second);
+			m_fscp_server->set_presentation_max_per_second(m_configuration.fscp.max_unauthenticated_messages_per_second);
 
 			m_fscp_server->set_hello_message_received_callback(boost::bind(&core::do_handle_hello_received, this, _1, _2));
 			m_fscp_server->set_contact_request_received_callback(boost::bind(&core::do_handle_contact_request_received, this, _1, _2, _3, _4));
@@ -669,10 +673,19 @@ namespace freelan
 			// Let's open the server.
 			m_fscp_server->open(listen_endpoint);
 
+			if (m_configuration.fscp.upnp_enabled)
+			{
+#ifdef USE_UPNP
+				m_fscp_server->upnp_punch_hole(listen_endpoint.port());
+#else
+				m_logger(fscp::log_level::error) << "UPnP support is not compiled in this version";
+#endif
+			}
+
 #ifdef LINUX
 			if (!m_configuration.fscp.listen_on_device.empty())
 			{
-				const auto socket_fd = m_fscp_server->get_socket().native();
+				const auto socket_fd = m_fscp_server->get_socket().native_handle();
 				const std::string device_name = m_configuration.fscp.listen_on_device;
 
 				if (::setsockopt(socket_fd, SOL_SOCKET, SO_BINDTODEVICE, device_name.c_str(), device_name.size()) == 0)
@@ -1766,19 +1779,24 @@ namespace freelan
 			{
 				m_logger(fscp::log_level::information) << "IPv4 address: " << m_configuration.tap_adapter.ipv4_address_prefix_length;
 
-				tap_config.ipv4.network_address = { m_configuration.tap_adapter.ipv4_address_prefix_length.address(), m_configuration.tap_adapter.ipv4_address_prefix_length.prefix_length() };
+				asiotap::ipv4_network_address address = { m_configuration.tap_adapter.ipv4_address_prefix_length.address(), m_configuration.tap_adapter.ipv4_address_prefix_length.prefix_length() };
+				tap_config.ipv4.network_address = address;
 			}
 			else
 			{
 				m_logger(fscp::log_level::information) << "No IPv4 address configured.";
 			}
 
+			// IPv4 DHCP (for Windows)
+			tap_config.ipv4.dhcp = m_configuration.tap_adapter.ipv4_dhcp;
+
 			// IPv6 address
 			if (!m_configuration.tap_adapter.ipv6_address_prefix_length.is_null())
 			{
 				m_logger(fscp::log_level::information) << "IPv6 address: " << m_configuration.tap_adapter.ipv6_address_prefix_length;
 
-				tap_config.ipv6.network_address = { m_configuration.tap_adapter.ipv6_address_prefix_length.address(), m_configuration.tap_adapter.ipv6_address_prefix_length.prefix_length() };
+				asiotap::ipv6_network_address address = { m_configuration.tap_adapter.ipv6_address_prefix_length.address(), m_configuration.tap_adapter.ipv6_address_prefix_length.prefix_length() };
+				tap_config.ipv6.network_address = address;
 			}
 			else
 			{
@@ -1968,6 +1986,7 @@ namespace freelan
 			m_tap_adapter->set_connected_state(false);
 
 			m_tap_adapter->close();
+			m_tap_adapter_io_service.stop();
 
 			m_tap_adapter_thread.join();
 		}
@@ -2040,16 +2059,21 @@ namespace freelan
 			return result;
 		}();
 
+		// disable buffer recycling from now (possible issue with SharedBuffer
+		// which references itself due to lambda capture)
 		m_tap_adapter->async_read(
 			buffer(receive_buffer),
 			boost::bind(
 				&core::do_handle_tap_adapter_read,
 				this,
+				/*
 				SharedBuffer(receive_buffer, [this](const SharedBuffer& buffer) {
 					m_tap_adapter_io_service.post([this, buffer] () {
 						m_tap_adapter_buffers.push_back(buffer);
 					});
 				}),
+				*/
+				receive_buffer,
 				boost::asio::placeholders::error,
 				boost::asio::placeholders::bytes_transferred
 			)
@@ -2355,6 +2379,11 @@ namespace freelan
 	{
 		if (m_configuration.server.enabled)
 		{
+#ifndef USE_MONGOOSE
+			// Due to GPL licensing issues, we can't include that in the build without violating the GPLv3 license.
+			// If you still want that support, adds USE_MONGOOSE define to the build options.
+			m_logger(fscp::log_level::warning) << "Web server support is not compiled in this version.";
+#else
 			if (m_configuration.server.protocol == server_configuration::server_protocol_type::https)
 			{
 				bool generated = false;
@@ -2414,11 +2443,13 @@ namespace freelan
 			m_web_server_thread = boost::thread([this](){ m_web_server->run(); });
 
 			m_logger(fscp::log_level::information) << "Web server started.";
+#endif
 		}
 	}
 
 	void core::close_web_server()
 	{
+#ifdef USE_MONGOOSE
 		if (m_web_server)
 		{
 			m_logger(fscp::log_level::information) << "Closing web server...";
@@ -2429,6 +2460,7 @@ namespace freelan
 
 			m_logger(fscp::log_level::information) << "Web server closed.";
 		}
+#endif
 	}
 
 	void core::open_web_client()
@@ -2521,6 +2553,14 @@ namespace freelan
 						open_fscp_server();
 					}
 
+					if(!m_web_client)
+					{
+						m_logger(fscp::log_level::information) << "FSCP closing as the web client was shut down.";
+						// closing core when FSCP was not setup so close it now!
+						close_fscp_server();
+						return;
+					}
+
 					const auto renew_timestamp = certificate.not_after().to_ptime() - RENEW_CERTIFICATE_WARNING_PERIOD;
 
 					m_logger(fscp::log_level::information) << "Certificate expires on " << local_adjustor::utc_to_local(certificate.not_after().to_ptime()) << ". Renewing on " << local_adjustor::utc_to_local(renew_timestamp) << ".";
@@ -2708,14 +2748,14 @@ namespace freelan
 				{
 					std::ostringstream oss;
 
-					for (auto&& endpoint : public_endpoints)
+					for (auto&& ep : public_endpoints)
 					{
 						if (!oss.str().empty())
 						{
 							oss << ", ";
 						}
 
-						oss << endpoint;
+						oss << ep;
 					}
 
 					m_logger(fscp::log_level::information) << "Setting contact information on the web server with " << public_endpoints.size() << " public endpoint(s) (" << oss.str() << ")...";
@@ -2747,14 +2787,14 @@ namespace freelan
 						{
 							std::ostringstream oss;
 
-							for (auto&& endpoint : accepted_endpoints)
+							for (auto&& ep : accepted_endpoints)
 							{
 								if (!oss.str().empty())
 								{
 									oss << ", ";
 								}
 
-								oss << endpoint;
+								oss << ep;
 							}
 
 							m_logger(fscp::log_level::information) << "Server will advertise the following endpoints: " << oss.str();
@@ -2764,14 +2804,14 @@ namespace freelan
 						{
 							std::ostringstream oss;
 
-							for (auto&& endpoint : rejected_endpoints)
+							for (auto&& ep : rejected_endpoints)
 							{
 								if (!oss.str().empty())
 								{
 									oss << ", ";
 								}
 
-								oss << endpoint;
+								oss << ep;
 							}
 
 							m_logger(fscp::log_level::warning) << "Server refused to advertise the following endpoints: " << oss.str();
@@ -2843,16 +2883,16 @@ namespace freelan
 							{
 								std::ostringstream oss;
 
-								for (auto&& endpoint : contact.second)
+								for (auto&& ep : contact.second)
 								{
-									async_contact(endpoint);
+									async_contact(ep);
 
 									if (!oss.str().empty())
 									{
 										oss << ", ";
 									}
 
-									oss << endpoint;
+									oss << ep;
 								}
 
 								m_logger(fscp::log_level::information) << "Contact information for " << contact.first << ": " << oss.str();
