@@ -1,5 +1,5 @@
 /*
- * libfscp - C++ portable OpenSSL cryptographic wrapper library.
+ * libfscp - A C++ library to establish peer-to-peer virtual private networks.
  * Copyright (C) 2010-2011 Julien Kauffmann <julien.kauffmann@freelan.org>
  *
  * This file is part of libfscp.
@@ -53,6 +53,10 @@
 #include "peer_session.hpp"
 #include "logger.hpp"
 
+#ifdef USE_UPNP
+#include "miniupnpcplus/upnp_device.hpp"
+#endif
+
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
 #include <boost/optional.hpp>
@@ -355,10 +359,18 @@ namespace fscp
 			 */
 			void close();
 
+#ifdef USE_UPNP
+			/**
+			 * \brief Use UPnP to punch hole NAT.
+			 * \param port local port.
+			 */
+			void upnp_punch_hole(uint16_t port);
+#endif
+
 			/**
 			 * \brief Greet an host.
 			 * \param target The target to greet.
-			 * \param handler The handler to call when a reply was received, an error occured or the request timed out.
+			 * \param handler The handler to call when a reply was received, an error occurred or the request timed out.
 			 * \param timeout The maximum time to wait for a reply.
 			 */
 			void async_greet(const ep_type& target, duration_handler_type handler, const boost::posix_time::time_duration& timeout = boost::posix_time::seconds(3));
@@ -409,6 +421,15 @@ namespace fscp
 				m_hello_message_received_handler = callback;
 			}
 
+			/**
+			 * \brief Set maximum hello message from one host per second.
+			 * \param max_per_second value to set.
+			 */
+			void set_hello_max_per_second(size_t max_per_second)
+			{
+				m_hello_max_per_second = max_per_second;
+			}
+
 			/**
 			 * \brief Set the hello message received callback.
 			 * \param callback The callback.
@@ -430,7 +451,7 @@ namespace fscp
 			/**
 			 * \brief Send a presentation message to the specified target.
 			 * \param target The target host.
-			 * \param handler The handler to call when the presentation message is sent or an error occured.
+			 * \param handler The handler to call when the presentation message is sent or an error occurred.
 			 */
 			void async_introduce_to(const ep_type& target, simple_handler_type handler);
 
@@ -445,7 +466,7 @@ namespace fscp
 
 			/**
 			 * \brief Resend a presentation message to all the known peers.
-			 * \param handler The handler to call when all the presentation messages were sent or an error occured.
+			 * \param handler The handler to call when all the presentation messages were sent or an error occurred.
 			 */
 			void async_reintroduce_to_all(multiple_endpoints_handler_type handler);
 
@@ -541,6 +562,15 @@ namespace fscp
 				m_presentation_message_received_handler = callback;
 			}
 
+			/**
+			 * \brief Set maximum presentation message from one host per second.
+			 * \param max_per_second value to set.
+			 */
+			void set_presentation_max_per_second(size_t max_per_second)
+			{
+				m_presentation_max_per_second = max_per_second;
+			}
+
 			/**
 			 * \brief Set the presentation message received callback.
 			 * \param callback The callback.
@@ -562,7 +592,7 @@ namespace fscp
 			/**
 			 * \brief Sends a session request to an host.
 			 * \param target The target to greet.
-			 * \param handler The handler to call when the request was sent or an error occured.
+			 * \param handler The handler to call when the request was sent or an error occurred.
 			 */
 			void async_request_session(const ep_type& target, simple_handler_type handler);
 
@@ -686,7 +716,7 @@ namespace fscp
 			 */
 			void set_elliptic_curves(const elliptic_curve_list_type& elliptic_curves)
 			{
-				m_elliptic_curves = elliptic_curves;
+				m_elliptic_curves = get_supported_elliptic_curves(elliptic_curves);
 			}
 
 			/**
@@ -908,7 +938,7 @@ namespace fscp
 			 * \param target The target host.
 			 * \param channel_number The channel number.
 			 * \param data The data to send.
-			 * \param handler The handler to call when the data was sent or an error occured.
+			 * \param handler The handler to call when the data was sent or an error occurred.
 			 */
 			void async_send_data(const ep_type& target, channel_number_type channel_number, boost::asio::const_buffer data, simple_handler_type handler);
 
@@ -928,7 +958,7 @@ namespace fscp
 			 * \param targets The list of hosts.
 			 * \param channel_number The channel number.
 			 * \param data The data to send.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			void async_send_data_to_list(const std::set<ep_type>& targets, channel_number_type channel_number, boost::asio::const_buffer data, multiple_endpoints_handler_type handler);
 
@@ -938,7 +968,7 @@ namespace fscp
 			 * \param end An iterator past the last endpoint of the list.
 			 * \param channel_number The channel number.
 			 * \param data The data to send.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			template <typename EPIterator>
 			void async_send_data_to_list(EPIterator begin, EPIterator end, channel_number_type channel_number, boost::asio::const_buffer data, multiple_endpoints_handler_type handler)
@@ -977,7 +1007,7 @@ namespace fscp
 			 * \brief Send data to all hosts.
 			 * \param channel_number The channel number.
 			 * \param data The data to send.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			void async_send_data_to_all(channel_number_type channel_number, boost::asio::const_buffer data, multiple_endpoints_handler_type handler)
 			{
@@ -998,7 +1028,7 @@ namespace fscp
 			 * \brief Send a contact request to an host.
 			 * \param target The target host.
 			 * \param hash_list The hashes to request.
-			 * \param handler The handler to call when the data was sent or an error occured.
+			 * \param handler The handler to call when the data was sent or an error occurred.
 			 */
 			void async_send_contact_request(const ep_type& target, const hash_list_type& hash_list, simple_handler_type handler);
 
@@ -1016,7 +1046,7 @@ namespace fscp
 			 * \brief Send a contact request to a list of hosts.
 			 * \param targets The list of hosts.
 			 * \param hash_list The hashes to request.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			void async_send_contact_request_to_list(const std::set<ep_type>& targets, const hash_list_type& hash_list, multiple_endpoints_handler_type handler);
 
@@ -1025,7 +1055,7 @@ namespace fscp
 			 * \param begin An iterator to the first endpoint of the list.
 			 * \param end An iterator past the last endpoint of the list.
 			 * \param hash_list The hashes to request.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			template <typename EPIterator>
 			void async_send_contact_request_to_list(EPIterator begin, EPIterator end, const hash_list_type& hash_list, multiple_endpoints_handler_type handler)
@@ -1061,7 +1091,7 @@ namespace fscp
 			/**
 			 * \brief Send a contact request to all hosts.
 			 * \param hash_list The hashes to request.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			void async_send_contact_request_to_all(const hash_list_type& hash_list, multiple_endpoints_handler_type handler)
 			{
@@ -1081,7 +1111,7 @@ namespace fscp
 			 * \brief Send contact information to an host.
 			 * \param target The target host.
 			 * \param contact_map The contact information.
-			 * \param handler The handler to call when the data was sent or an error occured.
+			 * \param handler The handler to call when the data was sent or an error occurred.
 			 */
 			void async_send_contact(const ep_type& target, const contact_map_type& contact_map, simple_handler_type handler);
 
@@ -1099,7 +1129,7 @@ namespace fscp
 			 * \brief Send contact information to a list of hosts.
 			 * \param targets The list of hosts.
 			 * \param contact_map The contact information.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			void async_send_contact_to_list(const std::set<ep_type>& targets, const contact_map_type& contact_map, multiple_endpoints_handler_type handler);
 
@@ -1108,7 +1138,7 @@ namespace fscp
 			 * \param begin An iterator to the first endpoint of the list.
 			 * \param end An iterator past the last endpoint of the list.
 			 * \param contact_map The contact information.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			template <typename EPIterator>
 			void async_send_contact_to_list(EPIterator begin, EPIterator end, const contact_map_type& contact_map, multiple_endpoints_handler_type handler)
@@ -1144,7 +1174,7 @@ namespace fscp
 			/**
 			 * \brief Send contact information to all hosts.
 			 * \param contact_map The contact information.
-			 * \param handler The handler to call when the data was sent to all hosts or an error occured.
+			 * \param handler The handler to call when the data was sent to all hosts or an error occurred.
 			 */
 			void async_send_contact_to_all(const contact_map_type& contact_map, multiple_endpoints_handler_type handler)
 			{
@@ -1255,6 +1285,7 @@ namespace fscp
 			void do_set_identity(const identity_store&, void_handler_type);
 
 		private:
+			elliptic_curve_list_type get_supported_elliptic_curves(const elliptic_curve_list_type& curves);
 
 			void async_receive_from()
 			{
@@ -1281,9 +1312,14 @@ namespace fscp
 			void pop_write();
 
 			socket_type m_socket;
+#if BOOST_ASIO_VERSION >= 101200 // Boost 1.66+
+			boost::asio::io_context::strand m_socket_strand;
+			boost::asio::io_context::strand m_write_queue_strand;
+#else
 			boost::asio::strand m_socket_strand;
-			std::queue<void_handler_type> m_write_queue;
 			boost::asio::strand m_write_queue_strand;
+#endif
+			std::queue<void_handler_type> m_write_queue;
 			std::list<SharedBuffer> m_socket_buffers;
 
 		private: // HELLO messages
@@ -1386,11 +1422,36 @@ namespace fscp
 			void do_set_accept_hello_messages_default(bool, void_handler_type);
 			void do_set_hello_message_received_callback(hello_message_received_handler_type, void_handler_type);
 
+			/**
+			 * \brief Reset hello limit.
+			 * \param ec error code.
+			 */
+			void do_hello_reset_limit(const boost::system::error_code& ec);
+
 			ep_hello_context_map m_ep_hello_contexts;
+#if BOOST_ASIO_VERSION >= 101200 // Boost 1.66+
+			boost::asio::io_context::strand m_greet_strand;
+#else
 			boost::asio::strand m_greet_strand;
+#endif
 			bool m_accept_hello_messages_default;
 			hello_message_received_handler_type m_hello_message_received_handler;
 
+			/**
+			 * \brief Current number of session request sent per endpoint.
+			 */
+			std::map<ep_type, size_t> m_hello_requests_map;
+
+			/**
+			 * \brief Timer for reesting hello requests limit.
+			 */
+			boost::asio::deadline_timer m_hello_limit_timer;
+
+			/**
+			 * \brief Maximum hello message from one host per second.
+			 */
+			size_t m_hello_max_per_second;
+
 		private: // PRESENTATION messages
 
 			typedef std::map<ep_type, presentation_store> presentation_store_map;
@@ -1406,11 +1467,36 @@ namespace fscp
 
 			void do_set_presentation_message_received_callback(presentation_message_received_handler_type, void_handler_type);
 
+			/**
+			 * \brief Reset presentation limit.
+			 * \param ec error code.
+			 */
+			void do_presentation_reset_limit(const boost::system::error_code& ec);
+
 			// This strand is also used by session requests and session messages during the cipherment/decipherment phase.
+#if BOOST_ASIO_VERSION >= 101200 // Boost 1.66+
+			boost::asio::io_context::strand m_presentation_strand;
+#else
 			boost::asio::strand m_presentation_strand;
+#endif
 			presentation_store_map m_presentation_store_map;
 			presentation_message_received_handler_type m_presentation_message_received_handler;
 
+			/**
+			 * \brief Current number of session request sent per endpoint.
+			 */
+			std::map<ep_type, size_t> m_presentation_requests_map;
+
+			/**
+			 * \brief Timer for reesting presentation requests limit.
+			 */
+			boost::asio::deadline_timer m_presentation_limit_timer;
+
+			/**
+			 * \brief Maximum presentation message from one host per second.
+			 */
+			size_t m_presentation_max_per_second;
+
 		private: // SESSION_REQUEST messages
 
 			typedef std::map<ep_type, peer_session> peer_session_map_type;
@@ -1433,7 +1519,11 @@ namespace fscp
 			void do_set_session_request_message_received_callback(session_request_received_handler_type, void_handler_type);
 
 			// This strand is common to session requests, session messages and data messages.
+#if BOOST_ASIO_VERSION >= 101200 // Boost 1.66+
+			boost::asio::io_context::strand m_session_strand;
+#else
 			boost::asio::strand m_session_strand;
+#endif
 
 			peer_session_map_type m_peer_sessions;
 			std::list<SharedBuffer> m_session_buffers;
@@ -1487,7 +1577,11 @@ namespace fscp
 			void do_set_contact_request_received_callback(contact_request_received_handler_type, void_handler_type);
 			void do_set_contact_received_callback(contact_received_handler_type, void_handler_type);
 
+#if BOOST_ASIO_VERSION >= 101200 // Boost 1.66+
+			boost::asio::io_context::strand m_contact_strand;
+#else
 			boost::asio::strand m_contact_strand;
+#endif
 
 			data_received_handler_type m_data_received_handler;
 			contact_request_received_handler_type m_contact_request_message_received_handler;
@@ -1502,6 +1596,10 @@ namespace fscp
 
 		private: // Misc
 
+#ifdef USE_UPNP
+			boost::shared_ptr<miniupnpcplus::upnp_device> m_upnp;
+#endif
+
 			friend std::ostream& operator<<(std::ostream& os, presentation_status_type status)
 			{
 				switch (status)
