@@ -1,5 +1,5 @@
 /*
- * libfscp - C++ portable OpenSSL cryptographic wrapper library.
+ * libfscp - A C++ library to establish peer-to-peer virtual private networks.
  * Copyright (C) 2010-2011 Julien Kauffmann <julien.kauffmann@freelan.org>
  *
  * This file is part of libfscp.
@@ -53,7 +53,16 @@
 #include "session_message.hpp"
 #include "data_message.hpp"
 
+// Mac OSX version of Boost has shadowing warnings that fail the build.
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wshadow"
+#endif
 #include <boost/random.hpp>
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
+
 #include <boost/make_shared.hpp>
 #include <boost/ref.hpp>
 #include <boost/thread/future.hpp>
@@ -225,12 +234,16 @@ namespace fscp
 		m_greet_strand(io_service),
 		m_accept_hello_messages_default(true),
 		m_hello_message_received_handler(),
+		m_hello_limit_timer(io_service, boost::posix_time::seconds(10)),
+		m_hello_max_per_second(1),
 		m_presentation_strand(io_service),
 		m_presentation_message_received_handler(),
+		m_presentation_limit_timer(io_service, boost::posix_time::seconds(10)),
+		m_presentation_max_per_second(1),
 		m_session_strand(io_service),
 		m_accept_session_request_messages_default(true),
 		m_cipher_suites(get_default_cipher_suites()),
-		m_elliptic_curves(get_default_elliptic_curves()),
+		m_elliptic_curves(get_supported_elliptic_curves(get_default_elliptic_curves())),
 		m_session_request_message_received_handler(),
 		m_accept_session_messages_default(true),
 		m_session_message_received_handler(),
@@ -248,6 +261,28 @@ namespace fscp
 		server_category();
 	}
 
+	elliptic_curve_list_type server::get_supported_elliptic_curves(
+			const elliptic_curve_list_type& curves)
+	{
+		elliptic_curve_list_type ret;
+
+		for (auto&& ec : curves)
+		{
+			try
+			{
+				cryptoplus::pkey::ecdhe_context(ec.to_elliptic_curve_nid())
+					.get_public_key();
+				ret.push_back(ec);
+			}
+			catch(boost::system::system_error&)
+			{
+				m_logger(log_level::warning) << "Elliptic curve not supported: "
+					<< ec.to_string();
+			}
+		}
+		return ret;
+	}
+
 	identity_store server::sync_get_identity()
 	{
 		typedef boost::promise<identity_store> promise_type;
@@ -285,6 +320,12 @@ namespace fscp
 		async_receive_from();
 
 		m_keep_alive_timer.async_wait(m_session_strand.wrap(boost::bind(&server::do_check_keep_alive, this, boost::asio::placeholders::error)));
+		m_hello_limit_timer.async_wait(m_greet_strand.wrap(
+					boost::bind(&server::do_hello_reset_limit, this,
+						boost::asio::placeholders::error)));
+		m_presentation_limit_timer.async_wait(m_presentation_strand.wrap(
+					boost::bind(&server::do_presentation_reset_limit, this,
+						boost::asio::placeholders::error)));
 	}
 
 	void server::close()
@@ -293,9 +334,45 @@ namespace fscp
 
 		m_keep_alive_timer.cancel();
 
+		m_hello_limit_timer.cancel();
+		m_presentation_limit_timer.cancel();
+
 		m_socket.close();
 	}
 
+#ifdef USE_UPNP
+	void server::upnp_punch_hole(uint16_t port)
+	{
+		try
+		{
+			std::ostringstream oss;
+			oss << port;
+
+			if(m_upnp.get() == nullptr)
+			{
+				m_logger(log_level::information) << "Discovering UPnP IGD gateways.";
+				m_upnp.reset(new miniupnpcplus::upnp_device(2000));
+
+				// same external port as local port
+				m_logger(log_level::trace) << "Try to register UPnP port mapping: " <<
+					m_upnp->get_external_ip() << ":" << port << " -> " << m_upnp->get_lan_ip() <<
+					":" << port;
+				m_upnp->register_port_mapping(miniupnpcplus::UDP, oss.str(), oss.str(),
+						"FreeLAN peer");
+				m_logger(log_level::information) << "UPnP port mapping registered: " <<
+					m_upnp->get_external_ip() << ":" << port << " -> " << m_upnp->get_lan_ip() <<
+					":" << port;
+
+				// TODO retry if port is already taken
+			}
+		}
+		catch (const boost::system::system_error& ex)
+		{
+			m_logger(log_level::error) << "UPnP discovery/port mapping failed: " << ex.what();
+		}
+	}
+#endif
+
 	void server::async_greet(const ep_type& target, duration_handler_type handler, const boost::posix_time::time_duration& timeout)
 	{
 		m_greet_strand.post(boost::bind(&server::do_greet, this, normalize(target), handler, timeout));
@@ -363,7 +440,7 @@ namespace fscp
 
 		if (item != m_presentation_store_map.end())
 		{
-			return boost::make_optional<presentation_store>(item->second);
+			return boost::optional<presentation_store>(item->second);
 		}
 		else
 		{
@@ -827,11 +904,14 @@ namespace fscp
 				this,
 				get_identity(),
 				sender,
+				/*
 				SharedBuffer(receive_buffer, [this](const SharedBuffer& buffer) {
 					m_socket_strand.post([this, buffer]() {
 						m_socket_buffers.push_back(buffer);
 					});
 				}),
+				*/
+				receive_buffer,
 				boost::asio::placeholders::error,
 				boost::asio::placeholders::bytes_transferred
 			)
@@ -1190,6 +1270,27 @@ namespace fscp
 	void server::do_handle_hello_request(const ep_type& sender, uint32_t hello_unique_number)
 	{
 		// All do_handle_hello_request() calls are done in the same strand so the following is thread-safe.
+		std::map<ep_type, size_t>::iterator sender_hello = m_hello_requests_map.find(sender);
+
+		if(sender_hello == m_hello_requests_map.end())
+		{
+			// new hello
+			m_hello_requests_map[sender] = 1;
+		}
+		else if(m_hello_requests_map[sender] >= (m_hello_max_per_second * 10))
+		{
+			// in 10s we have reach limits! Presentation flood?
+			m_logger(log_level::warning) <<
+				"Received too many HELLO messages from " << sender <<
+				", limit is " << (m_hello_max_per_second * 10) <<
+				" messages per 10 seconds";
+			return;
+		}
+		else
+		{
+			m_hello_requests_map[sender]++;
+		}
+
 		bool can_reply = m_accept_hello_messages_default;
 
 		if (m_hello_message_received_handler)
@@ -1230,6 +1331,21 @@ namespace fscp
 		}
 	}
 
+	void server::do_hello_reset_limit(const boost::system::error_code& ec)
+	{
+		// All do_hello_reset_limit calls are done in the same strand so the following is thread-safe.
+		if (ec != boost::asio::error::operation_aborted)
+		{
+			m_hello_requests_map.clear();
+
+			// rearm timer again
+			m_hello_limit_timer.expires_from_now(boost::posix_time::seconds(10));
+			m_hello_limit_timer.async_wait(m_greet_strand.wrap(
+						boost::bind(&server::do_hello_reset_limit, this,
+							boost::asio::placeholders::error)));
+		}
+	}
+
 	void server::do_set_hello_message_received_callback(hello_message_received_handler_type callback, void_handler_type handler)
 	{
 		// All do_set_hello_message_received_callback() calls are done in the same strand so the following is thread-safe.
@@ -1357,6 +1473,27 @@ namespace fscp
 	void server::do_handle_presentation(const identity_store& identity, const ep_type& sender, bool has_session, cert_type signature_certificate)
 	{
 		// All do_handle_presentation() calls are done in the same strand so the following is thread-safe.
+		std::map<ep_type, size_t>::iterator sender_presentation = m_presentation_requests_map.find(sender);
+
+		if(sender_presentation == m_presentation_requests_map.end())
+		{
+			// new presentation
+			m_presentation_requests_map[sender] = 1;
+		}
+		else if(m_presentation_requests_map[sender] >= (m_presentation_max_per_second * 10))
+		{
+			// in 10s we have reach limits! Presentation flood?
+			m_logger(log_level::warning) <<
+				"Received too many PRESENTATION messages from " << sender <<
+				", limit is " << (m_presentation_max_per_second * 10) <<
+				" messages per 10 seconds";
+			return;
+		}
+		else
+		{
+			m_presentation_requests_map[sender]++;
+		}
+
 		presentation_status_type presentation_status = PS_FIRST;
 
 		const presentation_store_map::iterator entry = m_presentation_store_map.find(sender);
@@ -1384,6 +1521,21 @@ namespace fscp
 		m_presentation_store_map[sender] = presentation_store(signature_certificate, identity.pre_shared_key());
 	}
 
+	void server::do_presentation_reset_limit(const boost::system::error_code& ec)
+	{
+		// All do_presentation_reset_limit calls are done in the same strand so the following is thread-safe.
+		if (ec != boost::asio::error::operation_aborted)
+		{
+			m_presentation_requests_map.clear();
+
+			// rearm timer again
+			m_presentation_limit_timer.expires_from_now(boost::posix_time::seconds(10));
+			m_presentation_limit_timer.async_wait(m_presentation_strand.wrap(
+						boost::bind(&server::do_presentation_reset_limit, this,
+							boost::asio::placeholders::error)));
+		}
+	}
+
 	void server::do_set_presentation_message_received_callback(presentation_message_received_handler_type callback, void_handler_type handler)
 	{
 		// All do_set_presentation_message_received_callback() calls are done in the same strand so the following is thread-safe.
@@ -1453,11 +1605,12 @@ namespace fscp
 			const auto next_session_number = p_session.next_session_number();
 			const auto local_host_identifier = p_session.local_host_identifier();
 
-			m_logger(log_level::trace) << "Sending session request message to " << target << " (next_session_number: " << next_session_number << ", local_host_identifier: " << local_host_identifier << ")";
 			size_t size = 0;
 
 			if (!!identity.signature_key())
 			{
+				m_logger(log_level::trace) << "Sending session request message to " << target << " (next_session_number: " << next_session_number << ", local_host_identifier: " << local_host_identifier << ", asymmetric signature)";
+
 				size = session_request_message::write(
 					buffer_cast<uint8_t*>(send_buffer),
 					buffer_size(send_buffer),
@@ -1470,6 +1623,8 @@ namespace fscp
 			}
 			else
 			{
+				m_logger(log_level::trace) << "Sending session request message to " << target << " (next_session_number: " << next_session_number << ", local_host_identifier: " << local_host_identifier << ", HMAC signature)";
+
 				size = session_request_message::write(
 					buffer_cast<uint8_t*>(send_buffer),
 					buffer_size(send_buffer),
@@ -1482,12 +1637,17 @@ namespace fscp
 					);
 			}
 
+			// disable buffer recycling from now (possible issue with SharedBuffer
+			// which references itself due to lambda capture)
 			async_send_to(
+				/*
 				SharedBuffer(send_buffer, [this](const SharedBuffer& buffer) {
 					m_session_strand.post([this, buffer]() {
 						m_session_buffers.push_back(buffer);
 					});
 				}),
+				*/
+				send_buffer,
 				size,
 				target,
 				handler
@@ -1531,23 +1691,25 @@ namespace fscp
 		}
 
 		// We make sure the signatures matches.
-		bool check_ok = false;
-
 		if (!!m_presentation_store_map[sender].signature_certificate())
 		{
-			check_ok = _session_request_message.check_signature(m_presentation_store_map[sender].signature_certificate().public_key());
+			 if (!_session_request_message.check_signature(m_presentation_store_map[sender].signature_certificate().public_key()))
+			 {
+				 m_logger(log_level::trace) << "Received a SESSION_REQUEST from " << sender << " with an invalid asymmetric signature. Ignoring.";
+
+				 return;
+			 }
 		}
 		else
 		{
 			const auto psk = m_presentation_store_map[sender].pre_shared_key();
-			check_ok = _session_request_message.check_signature(buffer_cast<const uint8_t*>(psk), buffer_size(psk));
-		}
 
-		if (!check_ok)
-		{
-			m_logger(log_level::trace) << "Received a SESSION_REQUEST from " << sender << " with an invalid signature. Ignoring.";
+			if (!_session_request_message.check_signature(buffer_cast<const uint8_t*>(psk), buffer_size(psk)))
+			{
+				 m_logger(log_level::trace) << "Received a SESSION_REQUEST from " << sender << " with an invalid HMAC signature. Ignoring.";
 
-			return;
+				 return;
+			}
 		}
 
 		// The make_shared_buffer_handler() call below is necessary so that the reference to session_request_message remains valid.
@@ -1588,7 +1750,7 @@ namespace fscp
 		if ((calg == cipher_suite_type::unsupported) || (ec == elliptic_curve_type::unsupported))
 		{
 			// No suitable cipher and/or elliptic curve is available.
-			m_logger(log_level::trace) << "Received a SESSION_REQUEST from " << sender << " but can't agree on the cipher suite to use (Cipher algorithm: " << calg  << ". Elliptic curve: " << ec << "). Ignoring";
+			m_logger(log_level::warning) << "Received a SESSION_REQUEST from " << sender << " but can't agree on the cipher suite to use. Ignoring.";
 
 			return;
 		}
@@ -1769,12 +1931,17 @@ namespace fscp
 				);
 			}
 
+			// disable buffer recycling from now (possible issue with SharedBuffer
+			// which references itself due to lambda capture)
 			async_send_to(
+				/*
 				SharedBuffer(send_buffer, [this](const SharedBuffer& buffer) {
 					m_session_strand.post([this, buffer]() {
 						m_session_buffers.push_back(buffer);
 					});
 				}),
+				*/
+				send_buffer,
 				size,
 				target,
 				[] (const boost::system::error_code&) {}
@@ -1800,23 +1967,25 @@ namespace fscp
 		}
 
 		// We make sure the signatures matches.
-		bool check_ok = false;
-
 		if (!!m_presentation_store_map[sender].signature_certificate())
 		{
-			check_ok = _session_message.check_signature(m_presentation_store_map[sender].signature_certificate().public_key());
+			if (!_session_message.check_signature(m_presentation_store_map[sender].signature_certificate().public_key()))
+			{
+				m_logger(log_level::trace) << "Received a SESSION from " << sender << " with an invalid asymmetric signature. Ignoring.";
+
+				return;
+			}
 		}
 		else
 		{
 			const auto psk = m_presentation_store_map[sender].pre_shared_key();
-			check_ok = _session_message.check_signature(buffer_cast<const uint8_t*>(psk), buffer_size(psk));
-		}
 
-		if (!check_ok)
-		{
-			m_logger(log_level::trace) << "Received a SESSION from " << sender << " with an invalid signature. Ignoring.";
+			if (!_session_message.check_signature(buffer_cast<const uint8_t*>(psk), buffer_size(psk)))
+			{
+				m_logger(log_level::trace) << "Received a SESSION from " << sender << " with an invalid HMAC signature. Ignoring.";
 
-			return;
+				return;
+			}
 		}
 
 		m_session_strand.post(
@@ -2084,12 +2253,17 @@ namespace fscp
 				buffer_size(p_session.current_session().local_nonce_prefix)
 			);
 
+			// disable buffer recycling from now (possible issue with SharedBuffer
+			// which references itself due to lambda capture)
 			async_send_to(
+				/*
 				SharedBuffer(send_buffer, [this](const SharedBuffer& buffer) {
 					m_session_strand.post([this, buffer]() {
 						m_session_buffers.push_back(buffer);
 					});
 				}),
+				*/
+				send_buffer,
 				size,
 				target,
 				handler
@@ -2315,11 +2489,14 @@ namespace fscp
 			do_handle_data_message(
 				sender,
 				type,
+				/*
 				SharedBuffer(cleartext_buffer, [this] (const SharedBuffer& buffer) {
 					m_session_strand.post([this, buffer] () {
 						m_session_buffers.push_back(buffer);
 					});
 				}),
+				*/
+				cleartext_buffer,
 				buffer(cleartext_buffer, cleartext_len)
 			);
 		}
